#define PROCTBL	0x400000

/****************************************************************************
 * _yield(src_pid, dest_pid):
 *
 *   This function yields control from the source process (ID src_pid)
 *   to the destination process (ID dest_pid).
 *
 *   It works in three parts.
 *   The _yield function saves the current process's register state on
 *   the current process's stack.
 *   At that point, the process has exactly two elements of state that
 *   need to be saved: the stack pointer and the program counter (instruction
 *   pointer).
 *   All other registers can be restored from the stack.
 *
 *   Then _yield calls the _yield2 function.
 *   The act of calling _yield2 saves the program counter on the stack
 *   as a return address.
 *   That leaves one important piece of state: the stack pointer.
 *   _yield2 explicitly saves that, into the PROCTBL array (physical memory
 *   address 0x400000).
 *
 *   Finally, _yield2 loads the destination environment's stack pointer
 *   from PROCTBL.
 *   A simple "ret" instruction is enough to return control to the destination,
 *   since we ensured that the topmost item on the stack is a return address!
 *
 *   It may be worth drawing a picture for yourself to see that this works.
 *
 */
	
.text
.globl _yield

_yield:		movl 4(%esp), %eax	// Load src_pid into %eax
		movl 8(%esp), %ecx	// Load dest_pid into %ecx
		pushal			// Save registers

		// Call _yield2.
		// The 'call' instruction will save the next EIP on the
		// stack, so when _yield2 begins, %esp will point to the
		// address of the 'popal' instruction.
		call _yield2

		// Control resumes here when someone yields back to this process!
		popal			// Restore registers
		ret			// Return to caller


_yield2:	// Store %esp in PROCTBL[src_pid].
		// This saves the current process's state.
		shll $2, %eax
		addl $PROCTBL, %eax	// %eax := $PROCTBL + 4*src_pid
		movl %esp, 0(%eax)

		// Load %esp from PROCTBL[dest_pid].
		shll $2, %ecx
		addl $PROCTBL, %ecx	// %ecx := $PROCTBL + 4*dst_pid
		movl 0(%ecx), %esp

		// At this point, the stack pointer points to a return
		// address within the destination process.
		// Usually that's the address of 'popal', above.
		// We can return to the destination process simply by
		// executing a 'ret' instruction!
		ret
